package com.knauf.ejb.kuchenzutat;

import java.util.logging.Logger;

import javax.ejb.CreateException;
import javax.ejb.EntityContext;

/**
 * Diese Bean bildet eine Zutat fr den Kuchen ab. Werte sind: a) ID (Versuch
 * ohne Auto-Generierte Primary Keys ! b) Name der Zutat c) Menge
 * 
 * @author Wolfgang Knauf
 */
public abstract class ZutatBean implements javax.ejb.EntityBean
{
  /**Der Entity-Context der Bean */
  private EntityContext entityContext = null;

  /**Der Logger. */
  private static final Logger logger = Logger.getLogger(ZutatBean.class.getName());


  /**EntityContext abrufen.
   * @return Der EntityContext.
   */
  public javax.ejb.EntityContext getEntityContext()
  {
    return entityContext;
  }

  ////////////////////////////////////////////////////////////////////
  //          Methoden aus Interface EntityBean                     //
  ////////////////////////////////////////////////////////////////////

  /**Bean wird aktiviert.
   * @throws javax.ejb.EJBException
   * @throws java.rmi.RemoteException
   */
  public void ejbActivate() throws javax.ejb.EJBException, java.rmi.RemoteException
  {
    ZutatBean.logger.info("ejbActivate");
  }

  /**Bean wird geladen.
   * @throws javax.ejb.EJBException
   * @throws java.rmi.RemoteException
   */
  public void ejbLoad() throws javax.ejb.EJBException, java.rmi.RemoteException
  {
    ZutatBean.logger.info("ejbLoad");
  }

  /**Bean wird passiviert.
   * @throws javax.ejb.EJBException
   * @throws java.rmi.RemoteException
   */
  public void ejbPassivate() throws javax.ejb.EJBException, java.rmi.RemoteException
  {
    ZutatBean.logger.info("ejbPassivate");
  }

  /**Bean wird gelscht.
   * @throws javax.ejb.EJBException
   * @throws java.rmi.RemoteException
   */
  public void ejbRemove() throws javax.ejb.RemoveException, javax.ejb.EJBException, java.rmi.RemoteException
  {
    ZutatBean.logger.info("ejbRemove");
  }

  /**Bean wird gespeichert.
   * @throws javax.ejb.EJBException
   * @throws java.rmi.RemoteException
   */
  public void ejbStore() throws javax.ejb.EJBException, java.rmi.RemoteException
  {
    ZutatBean.logger.info("ejbStore");
  }

  /**Entity-Context wird zurckgesetzt.
   * @throws javax.ejb.EJBException
   * @throws java.rmi.RemoteException
   */
  public void unsetEntityContext() throws javax.ejb.EJBException, java.rmi.RemoteException
  {
    ZutatBean.logger.info("unsetEntityContext");
    this.entityContext = null;
  }

  /**Entity-Context wird gesetzt.
   * @throws javax.ejb.EJBException
   * @throws java.rmi.RemoteException
   */
  public void setEntityContext(EntityContext parm1) throws javax.ejb.EJBException, java.rmi.RemoteException
  {
    ZutatBean.logger.info("setEntityContext");
    this.entityContext = parm1;
  }

  //////////////////////////////////////////////////////////////////////////////////
  //                   Zugriffsmethoden fr die Felder                            //
  //////////////////////////////////////////////////////////////////////////////////

  /**ID der Zutat abrufen.
   * @return Die ID
   */
  public abstract Integer getId();

  /**Setzen der Zutat-ID 
   * @param int_Id Die neue ID
   */
  public abstract void setId(Integer int_Id);

  /**Abrufen des Zutat-Namens.
   * @return Der Zutat-Name
   */
  public abstract String getZutatName();

  /**Setzen des Zutat-Namens.
   * @param str_Name Der neue Name.
   */
  public abstract void setZutatName(String str_Name);

  /**Abrufen der Menge. 
   * @return Die Menge der Zutat ("100 Gramm", "2 Stck")
   */
  public abstract String getMenge();

  /**Setzen der Menge ("100 Gramm", "2 Stck")
   * @param str_Menge Neue Menge.
   */
  public abstract void setMenge(String str_Menge);

  /**Abrufen des Kuchens, zu dem die Zutat gehrt.
   * @return Der Kuchen der Zutat.
   */
  public abstract KuchenLocal getKuchen();

  /**Setzen des Kuchens, zu dem die Zutat gehrt.
   * @param kuchen Der Kuchen der Zutat.
   */
  public abstract void setKuchen(KuchenLocal kuchen);

  //////////////////////////////////////////////////////////////////////////////////
  //                          SELECT-Methoden                                     //
  //////////////////////////////////////////////////////////////////////////////////

  /**Ermitteln der maximalen Zutat-ID in Datenbank
   * Der EJB-QL-Befehl wird per Deploytool festgelegt.
   * (select max (....)
   * @return Maximale ZutatID aus Datenbank.
   */
  public abstract Integer ejbSelectMaxId() throws javax.ejb.FinderException;

  /**Von auen verwendbare Methode aus dem Home-Interface: Ermitteln der nchsten verfgbaren
   * Zutat-ID.
   * @return Die neue Zutat-ID, 1 grer als die maximal vorhandene ID. WEnn es keine gibt,
   * wird "1" zurckgeliefert.
   */
  public Integer ejbHomeGetNextId()
  {
    Integer intMaxID = null;
    try
    {
      intMaxID = this.ejbSelectMaxId();
    }
    catch (javax.ejb.FinderException fex)
    {
      throw new javax.ejb.EJBException("Konnte MaxID fr Zutat nicht ermitteln", fex);
    }

    if (intMaxID == null)
      intMaxID = new Integer(0);
    //Um 1 erhhen:
    intMaxID = new Integer(intMaxID.intValue() + 1);
    ZutatBean.logger.info("ejbHomeGetNextId: " + intMaxID);

    return intMaxID;
  }

  //////////////////////////////////////////////////////////////////////////////////
  //                          Weitere Methoden                                    //
  //////////////////////////////////////////////////////////////////////////////////

  /**Erzeugen eines Zutat-Objekts, wobei Name und Menge angegeben wird. Der Key wird in der MEthode
   * erzeugt.
   * @param str_Name Der Name der Zutat
   * @param str_Menge Die Menge der Zutat
   * @return Immer NULL.
   * @exception CreateException Fehler beim Erzeugen.
   */
  public Integer ejbCreateByNameMenge(String str_Name, String str_Menge) throws CreateException
  {
    ZutatBean.logger.info("ejbCreateByNameMenge: " + str_Name + ", " + str_Menge);
    this.setZutatName(str_Name);
    this.setMenge(str_Menge);
    this.setId(this.ejbHomeGetNextId());

    return null;
  }

  /**Erzeugen eines Zutat-Objekts, Name und Menge sind noch nicht gegeben. Der Primary Key wird in der Methode
   * erzeugt.
   * @return Die ID
   * @exception CreateException Fehler beim Erzeugen.
   */
  public Integer ejbCreate() throws CreateException
  {
    ZutatBean.logger.info("ejbCreate");
    this.setId(this.ejbHomeGetNextId());

    return null;
  }

  /**Nach dem Erzeugen eines Zutat-Objekts, wobei Name und Menge angegeben werden: nichts besonderes zu tun.
   * @param str_Name Der Name der Zutat.
   * @param str_Menge Menge der Zutat
   * @exception CreateException Fehler beim Erzeugen.
   */
  public void ejbPostCreateByNameMenge(String str_Name, String str_Menge) throws CreateException
  {
    ZutatBean.logger.info("ejbPostCreateByNameMenge: " + str_Name + ", " + str_Menge);
  }

  /**Nach dem Erzeugen eines Zutat-Objekts, Name und Menge sind noch nicht gegeben.
   * @exception CreateException Fehler beim Erzeugen.
   */
  public void ejbPostCreate() throws CreateException
  {
    ZutatBean.logger.info("ejbPostCreate");
  }
}
